@@ -1,11 +1,24 @@
-/* _NVRM_COPYRIGHT_BEGIN_
+/*
+ * SPDX-FileCopyrightText: Copyright (c) 2015-21 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-License-Identifier: MIT
  *
- * Copyright 2015 by NVIDIA Corporation.  All rights reserved.  All
- * information contained herein is proprietary and confidential to NVIDIA
- * Corporation.  Any use, reproduction, or disclosure without the written
- * permission of NVIDIA Corporation is prohibited.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * _NVRM_COPYRIGHT_END_
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <linux/module.h>
@@ -35,13 +48,8 @@
 #include "conftest.h"
 #include "nv-procfs.h"
 #include "nv-kthread-q.h"
-
 #include "nv-time.h"
-
-#include <linux/sched.h>
-#if defined(NV_LINUX_SCHED_SIGNAL_H_PRESENT)
-#include <linux/sched/signal.h>
-#endif
+#include "nv-lock.h"
 
 #if !defined(CONFIG_RETPOLINE)
 #include "nv-retpoline.h"
@@ -51,6 +59,9 @@
 
 #define NVKMS_LOG_PREFIX "nvidia-modeset: "
 
+static bool output_rounding_fix = true;
+module_param_named(output_rounding_fix, output_rounding_fix, bool, 0400);
+
 /* These parameters are used for fault injection tests.  Normally the defaults
  * should be used. */
 MODULE_PARM_DESC(fail_malloc, "Fail the Nth call to nvkms_alloc");
@@ -63,6 +74,10 @@ module_param_named(malloc_verbose, malloc_verbose, bool, 0400);
 
 static atomic_t nvkms_alloc_called_count;
 
+NvBool nvkms_output_rounding_fix(void)
+{
+    return output_rounding_fix;
+}
 
 #define NVKMS_SYNCPT_STUBS_NEEDED
 
@@ -70,110 +85,6 @@ static atomic_t nvkms_alloc_called_count;
  * NVKMS interface for nvhost unit for sync point APIs.
  *************************************************************************/
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #ifdef NVKMS_SYNCPT_STUBS_NEEDED
 /* Unsupported STUB for nvkms_syncpt APIs */
 NvBool nvkms_syncpt_op(
@@ -267,18 +178,6 @@ static inline int nvkms_read_trylock_pm_lock(void)
     return !down_read_trylock(&nvkms_pm_lock);
 }
 
-static inline int nvkms_read_lock_pm_lock_interruptible(void)
-{
-    while (!down_read_trylock(&nvkms_pm_lock)) {
-        if (signal_pending(current)) {
-            return -EINTR;
-        }
-        cond_resched();
-    }
-
-    return 0;
-}
-
 static inline void nvkms_read_lock_pm_lock(void)
 {
     down_read(&nvkms_pm_lock);
@@ -884,18 +783,10 @@ void* nvkms_get_per_open_data(int fd)
         return NULL;
     }
 
-#if defined(NV_FILE_HAS_INODE)
     if (filp->f_inode == NULL) {
         goto done;
     }
     rdev = filp->f_inode->i_rdev;
-#else
-    if ((filp->f_dentry == NULL) ||
-        (filp->f_dentry->d_inode == NULL)) {
-        goto done;
-    }
-    rdev = filp->f_dentry->d_inode->i_rdev;
-#endif
 
     if ((MAJOR(rdev) != NVKMS_MAJOR_DEVICE_NUMBER) ||
         (MINOR(rdev) != NVKMS_MINOR_DEVICE_NUMBER)) {
@@ -934,18 +825,10 @@ NvBool nvkms_fd_is_nvidia_chardev(int fd)
         return ret;
     }
 
-#if defined(NV_FILE_HAS_INODE)
     if (filp->f_inode == NULL) {
         goto done;
     }
     rdev = filp->f_inode->i_rdev;
-#else
-    if ((filp->f_dentry == NULL) ||
-        (filp->f_dentry->d_inode == NULL)) {
-        goto done;
-    }
-    rdev = filp->f_dentry->d_inode->i_rdev;
-#endif
 
     if (MAJOR(rdev) == NVKMS_MAJOR_DEVICE_NUMBER) {
         ret = NV_TRUE;
@@ -996,12 +879,11 @@ NvBool nvkms_allow_write_combining(void)
     return __rm_ops.system_info.allow_write_combining;
 }
 
+#if IS_ENABLED(CONFIG_BACKLIGHT_CLASS_DEVICE)
 /*************************************************************************
  * Implementation of sysfs interface to control backlight
  *************************************************************************/
 
-#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
-
 struct nvkms_backlight_device {
     NvU32 gpu_id;
     NvU32 display_id;
@@ -1056,21 +938,18 @@ static const struct backlight_ops nvkms_backlight_ops = {
     .update_status = nvkms_update_backlight_status,
     .get_brightness = nvkms_get_backlight_brightness,
 };
-#endif
-
+#endif /* IS_ENABLED(CONFIG_BACKLIGHT_CLASS_DEVICE) */
 
 struct nvkms_backlight_device*
 nvkms_register_backlight(NvU32 gpu_id, NvU32 display_id, void *drv_priv,
                          NvU32 current_brightness)
 {
-#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
-    char name[11];
+#if IS_ENABLED(CONFIG_BACKLIGHT_CLASS_DEVICE)
+    char name[18];
     struct backlight_properties props = {
         .brightness = current_brightness,
         .max_brightness = 100,
-# if defined(NV_BACKLIGHT_PROPERTIES_TYPE_PRESENT)
         .type = BACKLIGHT_RAW,
-# endif
     };
     nv_gpu_info_t *gpu_info = NULL;
     NvU32 gpu_count = 0;
@@ -1122,18 +1001,18 @@ done:
     return nvkms_bd;
 #else
     return NULL;
-#endif
+#endif /* IS_ENABLED(CONFIG_BACKLIGHT_CLASS_DEVICE) */
 }
 
 void nvkms_unregister_backlight(struct nvkms_backlight_device *nvkms_bd)
 {
-#if defined(NV_BACKLIGHT_DEVICE_REGISTER_PRESENT)
+#if IS_ENABLED(CONFIG_BACKLIGHT_CLASS_DEVICE)
     if (nvkms_bd->dev) {
         backlight_device_unregister(nvkms_bd->dev);
     }
 
     nvkms_free(nvkms_bd, sizeof(*nvkms_bd));
-#endif
+#endif /* IS_ENABLED(CONFIG_BACKLIGHT_CLASS_DEVICE) */
 }
 
 /*************************************************************************
@@ -1368,15 +1247,8 @@ void nvkms_sema_up(nvkms_sema_handle_t *sema)
 
 #if defined(CONFIG_PROC_FS)
 
-#if defined(NV_DEFINE_SINGLE_PROCFS_FILE)
 #define NV_DEFINE_SINGLE_NVKMS_PROCFS_FILE(name) \
-    NV_DEFINE_SINGLE_PROCFS_FILE(name, \
-                                 nvkms_read_lock_pm_lock_interruptible, \
-                                 nvkms_read_unlock_pm_lock)
-#else
-#define NV_DEFINE_SINGLE_NVKMS_PROCFS_FILE(name) \
-    NV_DEFINE_PROCFS_SINGLE_FILE(name)
-#endif
+    NV_DEFINE_SINGLE_PROCFS_FILE_READ_ONLY(name, nvkms_pm_lock)
 
 #define NVKMS_PROCFS_FOLDER "driver/nvidia-modeset"
 
@@ -1521,7 +1393,7 @@ static int nvkms_open(struct inode *inode, struct file *filp)
 {
     int status;
 
-    status = nvkms_read_lock_pm_lock_interruptible();
+    status = nv_down_read_interruptible(&nvkms_pm_lock);
     if (status != 0) {
         return status;
     }
@@ -1578,7 +1450,7 @@ static int nvkms_ioctl(struct inode *inode, struct file *filp,
         return -EFAULT;
     }
 
-    status = nvkms_read_lock_pm_lock_interruptible();
+    status = nv_down_read_interruptible(&nvkms_pm_lock);
     if (status != 0) {
         return status;
     }
@@ -1759,11 +1631,7 @@ module_init(nvkms_init);
 module_exit(nvkms_exit);
 
 #if defined(MODULE_LICENSE)
-
-
-
   MODULE_LICENSE("NVIDIA");
-
 #endif
 #if defined(MODULE_INFO)
   MODULE_INFO(supported, "external");
