@@ -220,7 +220,8 @@ static void uvm_destroy_vma_semaphore_pool(struct vm_area_struct *vma)
                va_range->node.start   == vma->vm_start &&
                va_range->node.end + 1 == vma->vm_end &&
                va_range->type == UVM_VA_RANGE_TYPE_SEMAPHORE_POOL);
-    uvm_mem_unmap_cpu(va_range->semaphore_pool.mem);
+
+    uvm_mem_unmap_cpu_user(va_range->semaphore_pool.mem);
 }
 
 // If a fault handler is not set, paths like handle_pte_fault in older kernels
@@ -556,7 +557,7 @@ static vm_fault_t uvm_vm_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
         if (do_sleep)
             uvm_tools_record_throttling_end(va_space, fault_addr, UVM_ID_CPU);
 
-        status = uvm_va_block_find_create(va_space, NULL, fault_addr, &va_block);
+        status = uvm_va_block_find_create_managed(va_space, fault_addr, &va_block);
         if (status != NV_OK) {
             UVM_ASSERT_MSG(status == NV_ERR_NO_MEMORY, "status: %s\n", nvstatusToString(status));
             break;
@@ -685,10 +686,11 @@ static void uvm_vm_open_semaphore_pool(struct vm_area_struct *vma)
         // If we forked, leave the parent vma alone.
         uvm_disable_vma(vma);
 
-        // uvm_disable_vma unmaps in the parent as well; remap it.
-        uvm_global_processor_mask_clear(&va_range->semaphore_pool.mem->mapped_on, UVM_GLOBAL_ID_CPU);
+        // uvm_disable_vma unmaps in the parent as well; clear the uvm_mem CPU
+        // user mapping metadata and then remap.
+        uvm_mem_unmap_cpu_user(va_range->semaphore_pool.mem);
 
-        status = uvm_mem_map_cpu(va_range->semaphore_pool.mem, origin_vma);
+        status = uvm_mem_map_cpu_user(va_range->semaphore_pool.mem, va_range->va_space, origin_vma);
         if (status != NV_OK) {
             UVM_DBG_PRINT("Failed to remap semaphore pool to CPU for parent after fork; status = %d (%s)",
                     status, nvstatusToString(status));
@@ -699,7 +701,7 @@ static void uvm_vm_open_semaphore_pool(struct vm_area_struct *vma)
         origin_vma->vm_private_data = NULL;
         origin_vma->vm_ops = &uvm_vm_ops_disabled;
         vma->vm_ops = &uvm_vm_ops_disabled;
-        uvm_mem_unmap_cpu(va_range->semaphore_pool.mem);
+        uvm_mem_unmap_cpu_user(va_range->semaphore_pool.mem);
     }
 
     uvm_va_space_up_write(va_space);
@@ -835,7 +837,7 @@ static int uvm_mmap(struct file *filp, struct vm_area_struct *vma)
     // 1) An overlapping range was previously created with an ioctl, for example
     //    for an external mapping.
     // 2) This file was passed to another process via a UNIX domain socket
-    status = uvm_va_range_create_mmap(va_space, vma->vm_private_data, NULL);
+    status = uvm_va_range_create_mmap(va_space, current->mm, vma->vm_private_data, NULL);
 
     if (status == NV_ERR_UVM_ADDRESS_IN_USE) {
         // If the mmap is for a semaphore pool, the VA range will have been
@@ -849,7 +851,7 @@ static int uvm_mmap(struct file *filp, struct vm_area_struct *vma)
             vma_wrapper_allocated = false;
             vma->vm_private_data = vma;
             vma->vm_ops = &uvm_vm_ops_semaphore_pool;
-            status = uvm_mem_map_cpu(va_range->semaphore_pool.mem, vma);
+            status = uvm_mem_map_cpu_user(va_range->semaphore_pool.mem, va_range->va_space, vma);
         }
     }
 
@@ -1092,17 +1094,6 @@ static int uvm_init(void)
     if (uvm_enable_builtin_tests)
         pr_info("Built-in UVM tests are enabled. This is a security risk.\n");
 
-
-
-
-
-
-
-
-
-
-
-
     return 0;
 
 error:
@@ -1144,4 +1135,4 @@ module_exit(uvm_exit_entry);
 
 MODULE_LICENSE("Dual MIT/GPL");
 MODULE_INFO(supported, "external");
-
+MODULE_VERSION(NV_VERSION_STRING);
