@@ -88,16 +88,8 @@
 
 #include "nv-kthread-q.h"
 
-
-
-
-
-
-
-
     #define UVM_THREAD_AFFINITY_SUPPORTED() 0
 
-
 // The ARM arch lacks support for cpumask_of_node() until kernel 4.7. It was
 // added via commit1a2db300348b ("arm64, numa: Add NUMA support for arm64
 // platforms.") Callers should either check UVM_THREAD_AFFINITY_SUPPORTED()
@@ -112,18 +104,8 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
 #endif
 }
 
-
-
-
-
-
-
-
-
-
     #define UVM_IS_CONFIG_HMM() 0
 
-
 // Various issues prevent us from using mmu_notifiers in older kernels. These
 // include:
 //  - ->release being called under RCU instead of SRCU: fixed by commit
@@ -139,13 +121,6 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
 //
 // The callback was added in commit 0f0a327fa12cd55de5e7f8c05a70ac3d047f405e,
 // v3.19 (2014-11-13).
-
-
-
-
-
-
-
     #if defined(NV_MMU_NOTIFIER_OPS_HAS_INVALIDATE_RANGE) && \
         defined(NVCPU_PPC64LE) && LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0)
         #define UVM_CAN_USE_MMU_NOTIFIERS() 1
@@ -153,7 +128,6 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
         #define UVM_CAN_USE_MMU_NOTIFIERS() 0
     #endif
 
-
 // See bug 1707453 for further details about setting the minimum kernel version.
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 32)
 #  error This driver does not support kernels older than 2.6.32!
@@ -234,7 +208,7 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
 #define __GFP_NORETRY 0
 #endif
 
-#define NV_UVM_GFP_FLAGS (GFP_KERNEL | __GFP_NORETRY)
+#define NV_UVM_GFP_FLAGS (GFP_KERNEL)
 
 #if !defined(NV_ADDRESS_SPACE_INIT_ONCE_PRESENT)
     void address_space_init_once(struct address_space *mapping);
@@ -285,16 +259,6 @@ static inline const struct cpumask *uvm_cpumask_of_node(int node)
 #define NV_ALIGN_DOWN(v,g) ((v) & ~((g) - 1))
 #endif
 
-#if !defined(NV_KBASENAME_PRESENT)
-static inline const char *kbasename(const char *str)
-{
-    const char *p = strrchr(str, '/');
-    if (!p)
-        return str;
-    return p + 1;
-}
-#endif
-
 #if defined(NVCPU_X86)
 /* Some old IA32 kernels don't have 64/64 division routines,
  * they only support 64/32 division with do_div(). */
@@ -549,4 +513,41 @@ static void __sched usleep_range(unsigned long min, unsigned long max)
 }
 #endif
 
+typedef struct
+{
+    struct mem_cgroup *new_memcg;
+    struct mem_cgroup *old_memcg;
+} uvm_memcg_context_t;
+
+    #define UVM_CGROUP_ACCOUNTING_SUPPORTED() 0
+    #define NV_UVM_GFP_FLAGS_ACCOUNT              (NV_UVM_GFP_FLAGS)
+
+    static inline void uvm_memcg_context_start(uvm_memcg_context_t *context, struct mm_struct *mm)
+    {
+        return;
+    }
+
+    static inline void uvm_memcg_context_end(uvm_memcg_context_t *context)
+    {
+        return;
+    }
+
+#if defined(NVCPU_X86) || defined(NVCPU_X86_64)
+  #include <asm/pgtable_types.h>
+#endif
+
+#if !defined(PAGE_KERNEL_NOENC)
+  #define PAGE_KERNEL_NOENC PAGE_KERNEL
+#endif
+
+// Commit 1dff8083a024650c75a9c961c38082473ceae8cf (v4.7).
+//
+// Archs with CONFIG_MMU should have their own page.h, and can't include
+// asm-generic/page.h. However, x86, powerpc, arm64 don't define page_to_virt()
+// macro in their version of page.h.
+#include <linux/mm.h>
+#ifndef page_to_virt
+  #include <asm/page.h>
+  #define page_to_virt(x)    __va(PFN_PHYS(page_to_pfn(x)))
+#endif
 #endif // _UVM_LINUX_H
