@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015-2021 NVIDIA Corporation
+    Copyright (c) 2015-2022 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -38,9 +38,12 @@
 #include "uvm_perf_thrashing.h"
 #include "uvm_perf_utils.h"
 #include "uvm_va_block_types.h"
+#include "uvm_range_tree.h"
 #include "uvm_mmu.h"
 #include "nv-kthread-q.h"
 
+#include <linux/mmu_notifier.h>
+
 // VA blocks are the leaf nodes in the uvm_va_space tree for managed allocations
 // (VA ranges with type == UVM_VA_RANGE_TYPE_MANAGED):
 //
@@ -107,14 +110,7 @@ typedef struct
     // Pages that have been evicted to sysmem
     uvm_page_mask_t evicted;
 
-    // Array of addresses of CPU pages mapped for physical access by the GPU.
-    //
-    // The CPU pages are mapped eagerly on the GPU whenever a new CPU page is
-    // allocated and whenever a new GPU state is allocated.
-    //
-    // The size of this array is always the same as the uvm_va_block::cpu.pages
-    // array.
-    NvU64 *cpu_pages_dma_addrs;
+    NvU64 *cpu_chunks_dma_addrs;
 
     // Array of naturally-aligned chunks. Each chunk has the largest possible
     // size which can fit within the block, so they are not uniform size.
@@ -258,14 +254,6 @@ struct uvm_va_block_struct
     // waiting for the last ref count to be removed have va_range and
     // hmm.va_space set to NULL (could be either type of block).
     //
-    // TODO: Bug 1765193. Since the va_range contains the policy for this block
-    //       (preferred location, accessed-by, etc) the parent va_range must be
-    //       inspected for operations on this block. That's ok for migrations
-    //       and GPU faults since they lock the va_space first, but it's not ok
-    //       for the eviction path which only locks the block. We'll have to
-    //       figure out how to handle that, perhaps by having block operations
-    //       take a snapshot of the range's policy state.
-    //
     // This field can be read while holding either the block lock or just the VA
     // space lock in read mode, since it can only change when the VA space lock
     // is held in write mode.
@@ -336,13 +324,18 @@ struct uvm_va_block_struct
         // overhead on the whole.
         uvm_page_mask_t resident;
 
-        // Per-page array of physical pages. This array scales dynamically with
-        // the block size.
+        // CPU memory chunks represent physically contiguous CPU memory
+        // allocations. See uvm_pmm_sysmem.h for more details on CPU chunks.
+        // This member is meant to hold an opaque value indicating the CPU
+        // chunk storage method. For more details on CPU chunk storage,
+        // see uvm_cpu_chunk_storage_type_t in uvm_pmm_sysmem.c.
+        unsigned long chunks;
+
+        // Per-page allocation bit vector.
         //
-        // Note that the pages need to be physically mapped on each GPU before
-        // being accessed and these mappings are tracked in the GPU state,
-        // uvm_va_block_gpu_state_t::cpu_pages_dma_addrs.
-        struct page **pages;
+        // A set bit means that a CPU page has been allocated for the
+        // corresponding page index.
+        uvm_page_mask_t allocated;
 
         // Per-page mapping bit vectors, one per bit we need to track. These are
         // used for fast traversal of valid mappings in the block. These contain
@@ -448,12 +441,22 @@ struct uvm_va_block_struct
     struct
     {
 
-
-
-
-
-        // Parent VA space pointer.
+        // Parent VA space pointer. It is NULL for UVM managed blocks or if
+        // the HMM block is dead. This field can be read while holding the
+        // block lock and is only modified while holding the va_space write
+        // lock and va_block lock (same as the va_range pointer).
         uvm_va_space_t *va_space;
+
+        // Tree of uvm_va_policy_node_t. The policy node ranges always cover
+        // all or part of a VMA range or a contiguous range of VMAs within the
+        // va_block. Policy nodes are resized or deleted when the underlying
+        // VMA range is changed by Linux via the invalidate() callback.
+        // Otherwise, policies could be stale after munmap().
+        // Locking: The va_block lock is needed to access or modify the tree.
+        uvm_range_tree_t va_policy_tree;
+
+        // Storage node for range tree of va_blocks.
+        uvm_range_tree_node_t node;
     } hmm;
 #endif
 };
@@ -477,6 +480,12 @@ struct uvm_va_block_wrapper_struct
         // with eviction enabled. Used for testing only.
         NvU32 user_pages_allocation_retry_force_count;
 
+        // Mask of chunk sizes to be used for CPU chunk allocations.
+        // The actual set of chunk sizes to be used will be the set resulting
+        // from AND'ing this value with the value of
+        // uvm_cpu_chunk_allocation_sizes module parameter.
+        NvU32 cpu_chunk_allocation_size_mask;
+
         // Force the next eviction attempt on this block to fail. Used for
         // testing only.
         bool inject_eviction_error;
@@ -538,6 +547,11 @@ static inline void uvm_va_block_retain(uvm_va_block_t *va_block)
     nv_kref_get(&va_block->kref);
 }
 
+// Locking: The va_block lock must not be held.
+// The va_space lock must be held in write mode unless it is the special case
+// that the block has no GPU state; for example, right after calling
+// uvm_va_block_create(). In that case, the va_space lock can be held in read
+// mode.
 static inline void uvm_va_block_release(uvm_va_block_t *va_block)
 {
     if (va_block) {
@@ -556,6 +570,19 @@ static inline void uvm_va_block_release_no_destroy(uvm_va_block_t *va_block)
     UVM_ASSERT(!destroyed);
 }
 
+// Returns true if the block is managed by HMM.
+// Locking: This can be called while holding either the block lock or just the
+// VA space lock in read mode, since it can only change when the VA space lock
+// is held in write mode.
+static inline bool uvm_va_block_is_hmm(uvm_va_block_t *va_block)
+{
+#if UVM_IS_CONFIG_HMM()
+    return va_block->hmm.va_space;
+#else
+    return false;
+#endif
+}
+
 // Return true if the block is dead.
 // Locking: This can be called while holding either the block lock or just the
 // VA space lock in read mode, since it can only change when the VA space lock
@@ -573,6 +600,11 @@ static inline bool uvm_va_block_is_dead(uvm_va_block_t *va_block)
     return true;
 }
 
+static inline uvm_va_block_gpu_state_t *uvm_va_block_gpu_state_get(uvm_va_block_t *va_block, uvm_gpu_id_t gpu_id)
+{
+    return va_block->gpus[uvm_id_gpu_index(gpu_id)];
+}
+
 // Return the va_space pointer of the given block or NULL if the block is dead.
 // Locking: This can be called while holding either the block lock or just the
 // VA space lock in read mode, since it can only change when the VA space lock
@@ -604,9 +636,6 @@ static void uvm_va_block_context_init(uvm_va_block_context_t *va_block_context,
     if (UVM_IS_DEBUG())
         memset(va_block_context, 0xff, sizeof(*va_block_context));
 
-    if (mm)
-        uvm_assert_mmap_lock_locked(mm);
-
     va_block_context->mm = mm;
 }
 
@@ -958,9 +987,11 @@ NV_STATUS uvm_va_block_add_gpu_va_space(uvm_va_block_t *va_block, uvm_gpu_va_spa
 // If mm != NULL, that mm is used for any CPU mappings which may be created as
 // a result of this call. See uvm_va_block_context_t::mm for details.
 //
-// LOCKING: The caller must hold the va_block lock. If mm != NULL, the caller
-//          must hold mm->mmap_lock in at least read mode.
-void uvm_va_block_remove_gpu_va_space(uvm_va_block_t *va_block, uvm_gpu_va_space_t *gpu_va_space, struct mm_struct *mm);
+// LOCKING: The caller must hold the va_block lock. If block_context->mm is not
+// NULL, the caller must hold mm->mmap_lock in at least read mode.
+void uvm_va_block_remove_gpu_va_space(uvm_va_block_t *va_block,
+                                      uvm_gpu_va_space_t *gpu_va_space,
+                                      uvm_va_block_context_t *block_context);
 
 // Creates any mappings necessary in this VA block between the two GPUs, in
 // either direction.
@@ -1021,6 +1052,18 @@ NV_STATUS uvm_va_block_split(uvm_va_block_t *existing_va_block,
                              uvm_va_block_t **new_va_block,
                              uvm_va_range_t *new_va_range);
 
+// Exactly the same split semantics as uvm_va_block_split, including error
+// handling except the existing_va_block block lock needs to be held and
+// the new_va_block has to be preallocated.
+//
+// new_va_block's va_range is set to new_va_range before any reverse mapping is
+// established to the new block, but the caller is responsible for inserting the
+// new block into the range.
+NV_STATUS uvm_va_block_split_locked(uvm_va_block_t *existing_va_block,
+                                    NvU64 new_end,
+                                    uvm_va_block_t *new_va_block,
+                                    uvm_va_range_t *new_va_range);
+
 // Handles a CPU fault in the given VA block, performing any operations
 // necessary to establish a coherent CPU mapping (migrations, cache invalidates,
 // etc.).
@@ -1133,10 +1176,10 @@ const uvm_page_mask_t *uvm_va_block_map_mask_get(uvm_va_block_t *block, uvm_proc
 //      uvm_va_block_find
 //      uvm_va_block_find_create
 
-// Finds the VA block containing addr, if any. The va_space->lock must be held
-// in at least read mode. Return values:
-// NV_ERR_INVALID_ADDRESS   addr is not covered by a va_range, or the va_range
-//                          does not have type UVM_VA_RANGE_TYPE_MANAGED
+// Finds the UVM or HMM VA block containing addr, if any. The va_space->lock
+// must be held in at least read mode. Return values:
+// NV_ERR_INVALID_ADDRESS   addr is not a UVM_VA_RANGE_TYPE_MANAGED va_range nor
+//                          a HMM enabled VMA.
 //
 // NV_ERR_OBJECT_NOT_FOUND  addr is valid but no block has been allocated to
 //                          cover it yet
@@ -1144,13 +1187,30 @@ const uvm_page_mask_t *uvm_va_block_map_mask_get(uvm_va_block_t *block, uvm_proc
 // NV_OK                    The block was returned successfully
 NV_STATUS uvm_va_block_find(uvm_va_space_t *va_space, NvU64 addr, uvm_va_block_t **out_block);
 
-// Same as uvm_va_block_find except that the block is created if not already
-// present in the VA range.
+// Same as uvm_va_block_find except that the block is created if not found.
+// If addr is covered by a UVM_VA_RANGE_TYPE_MANAGED va_range, a managed block
+// will be created. Otherwise, if addr is not covered by any va_range, mm is
+// non-NULL, and HMM is enabled in the va_space, an HMM block will be created.
+// In either case, if mm is non-NULL, it must be retained and locked in at
+// least read mode. Return values:
+// NV_ERR_INVALID_ADDRESS   addr is not a UVM_VA_RANGE_TYPE_MANAGED va_range nor
+//                          a HMM enabled VMA.
+// NV_ERR_NO_MEMORY         memory could not be allocated.
 NV_STATUS uvm_va_block_find_create(uvm_va_space_t *va_space,
                                    struct mm_struct *mm,
                                    NvU64 addr,
+                                   uvm_va_block_context_t *va_block_context,
                                    uvm_va_block_t **out_block);
 
+// Same as uvm_va_block_find_create except that only UVM managed va_blocks are
+// created if not already present in the VA range.
+static NV_STATUS uvm_va_block_find_create_managed(uvm_va_space_t *va_space,
+                                                  NvU64 addr,
+                                                  uvm_va_block_t **out_block)
+{
+    return uvm_va_block_find_create(va_space, NULL, addr, NULL, out_block);
+}
+
 // Look up a chunk backing a specific address within the VA block. Returns NULL if none.
 uvm_gpu_chunk_t *uvm_va_block_lookup_gpu_chunk(uvm_va_block_t *va_block, uvm_gpu_t *gpu, NvU64 address);
 
@@ -1190,7 +1250,11 @@ NV_STATUS uvm_va_block_migrate_locked(uvm_va_block_t *va_block,
 // The caller needs to support allocation-retry of page tables.
 //
 // LOCKING: The caller must hold the va_block lock
-NV_STATUS uvm_va_block_write_from_cpu(uvm_va_block_t *va_block, NvU64 dst, uvm_mem_t *src, size_t size);
+NV_STATUS uvm_va_block_write_from_cpu(uvm_va_block_t *va_block,
+                                      uvm_va_block_context_t *block_context,
+                                      NvU64 dst,
+                                      uvm_mem_t *src,
+                                      size_t size);
 
 // Read block's data into a CPU buffer
 //
@@ -1704,6 +1768,17 @@ static NvU16 uvm_va_block_bitmap_tree_iter_get_count(const uvm_va_block_bitmap_t
          (counter) = --(iter)->level_idx < 0? 0:                                                       \
                                               uvm_va_block_bitmap_tree_iter_get_count((tree), (iter)))
 
+// Return the block region covered by the given chunk size. page_index must be
+// any page within the block known to be covered by the chunk.
+static uvm_va_block_region_t uvm_va_block_chunk_region(uvm_va_block_t *block,
+                                                       uvm_chunk_size_t chunk_size,
+                                                       uvm_page_index_t page_index)
+{
+    NvU64 page_addr = uvm_va_block_cpu_page_address(block, page_index);
+    NvU64 chunk_start_addr = UVM_ALIGN_DOWN(page_addr, chunk_size);
+    uvm_page_index_t first = (uvm_page_index_t)((chunk_start_addr - block->start) / PAGE_SIZE);
+    return uvm_va_block_region(first, first + (chunk_size / PAGE_SIZE));
+}
 
 //
 // Helpers for page state (permissions, size, residency)
@@ -1834,6 +1909,7 @@ uvm_processor_id_t uvm_va_block_select_residency(uvm_va_block_t *va_block,
                                                  uvm_page_index_t page_index,
                                                  uvm_processor_id_t processor_id,
                                                  NvU32 access_type_mask,
+                                                 uvm_va_policy_t *policy,
                                                  const uvm_perf_thrashing_hint_t *thrashing_hint,
                                                  uvm_service_operation_t operation,
                                                  bool *read_duplicate);
