@@ -28,14 +28,6 @@
 
 #include "nv-time.h"
 
-
-
-
-
-
-
-
-
 extern char *NVreg_TemporaryFilePath;
 
 #define MAX_ERROR_STRING 512
@@ -52,16 +44,16 @@ NvU8  os_page_shift = PAGE_SHIFT;
 NvU32 os_sev_status = 0;
 NvBool os_sev_enabled = 0;
 
-NV_STATUS NV_API_CALL os_disable_console_access(void)
+NvBool os_dma_buf_enabled = NV_FALSE;
+
+void NV_API_CALL os_disable_console_access(void)
 {
-    NV_ACQUIRE_CONSOLE_SEM();
-    return NV_OK;
+    console_lock();
 }
 
-NV_STATUS NV_API_CALL os_enable_console_access(void)
+void NV_API_CALL os_enable_console_access(void)
 {
-    NV_RELEASE_CONSOLE_SEM();
-    return NV_OK;
+    console_unlock();
 }
 
 typedef struct semaphore os_mutex_t;
@@ -665,19 +657,6 @@ void NV_API_CALL os_get_current_process_name(char *buf, NvU32 len)
     task_unlock(current);
 }
 
-NvU32 NV_API_CALL os_get_current_pasid(void)
-{
-#if NVCPU_IS_PPC64LE && defined(NV_MM_CONTEXT_T_HAS_ID)
-    if (current && current->mm)
-        return current->mm->context.id;
-    else
-        return 0;
-#else
-    // Unsupported. Return "0" as it's invalid PASID
-    return 0;
-#endif
-}
-
 NV_STATUS NV_API_CALL os_get_current_thread(NvU64 *threadId)
 {
     if (in_interrupt())
@@ -878,7 +857,7 @@ void* NV_API_CALL os_map_kernel_space(
             vaddr = nv_ioremap_cache(start, size_bytes);
             break;
         case NV_MEMORY_WRITECOMBINED:
-            vaddr = rm_disable_iomap_wc() ? 
+            vaddr = rm_disable_iomap_wc() ?
                     nv_ioremap_nocache(start, size_bytes) :
                     nv_ioremap_wc(start, size_bytes);
             break;
@@ -979,6 +958,11 @@ void NV_API_CALL os_dbg_set_level(NvU32 new_debuglevel)
     cur_debuglevel = new_debuglevel;
 }
 
+NvU64 NV_API_CALL os_get_max_user_va(void)
+{
+	return TASK_SIZE;
+}
+
 NV_STATUS NV_API_CALL os_schedule(void)
 {
     if (NV_MAY_SLEEP())
@@ -1069,23 +1053,28 @@ NV_STATUS NV_API_CALL os_flush_work_queue(struct os_work_queue *queue)
     }
 }
 
+extern NvU32 NVreg_EnableDbgBreakpoint;
+
 void NV_API_CALL os_dbg_breakpoint(void)
 {
-#if defined(DEBUG)
-  #if defined(CONFIG_X86_REMOTE_DEBUG) || defined(CONFIG_KGDB) || defined(CONFIG_XMON)
-    #if defined(NVCPU_X86_64)
-        __asm__ __volatile__ ("int $3");
-    #elif defined(NVCPU_ARM)
-        __asm__ __volatile__ (".word %c0" :: "i" (KGDB_COMPILED_BREAK));
-    #elif defined(NVCPU_AARCH64)
-        # warning "Need to implement os_dbg_breakpoint() for aarch64"
-    #elif defined(NVCPU_PPC64LE)
-        __asm__ __volatile__ ("trap");
-    #endif // NVCPU_*
-  #elif defined(CONFIG_KDB)
-      KDB_ENTER();
-  #endif // CONFIG_X86_REMOTE_DEBUG || CONFIG_KGDB || CONFIG_XMON
-#endif // DEBUG
+    if (NVreg_EnableDbgBreakpoint == 0)
+    {
+        return;
+    }
+
+#if defined(CONFIG_X86_REMOTE_DEBUG) || defined(CONFIG_KGDB) || defined(CONFIG_XMON)
+  #if defined(NVCPU_X86_64)
+    __asm__ __volatile__ ("int $3");
+  #elif defined(NVCPU_ARM)
+    __asm__ __volatile__ (".word %c0" :: "i" (KGDB_COMPILED_BREAK));
+  #elif defined(NVCPU_AARCH64)
+    # warning "Need to implement os_dbg_breakpoint() for aarch64"
+  #elif defined(NVCPU_PPC64LE)
+    __asm__ __volatile__ ("trap");
+  #endif // NVCPU_*
+#elif defined(CONFIG_KDB)
+    KDB_ENTER();
+#endif // CONFIG_X86_REMOTE_DEBUG || CONFIG_KGDB || CONFIG_XMON
 }
 
 NvU32 NV_API_CALL os_get_cpu_number()
@@ -1120,28 +1109,58 @@ void NV_API_CALL os_get_screen_info(
     NvU64 consoleBar2Address
 )
 {
-#if defined(CONFIG_FB)
-    int i;
     *pPhysicalAddress = 0;
     *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
 
-    for (i = 0; i < num_registered_fb; i++)
+#if defined(CONFIG_FB) && defined(NV_NUM_REGISTERED_FB_PRESENT)
+    if (num_registered_fb > 0)
     {
-        /* Make sure base address is mapped to GPU BAR */
-        if ((registered_fb[i]->fix.smem_start == consoleBar1Address) ||
-            (registered_fb[i]->fix.smem_start == consoleBar2Address))
+        int i;
+
+        for (i = 0; i < num_registered_fb; i++)
         {
-            *pPhysicalAddress = registered_fb[i]->fix.smem_start;
-            *pFbWidth = registered_fb[i]->var.xres;
-            *pFbHeight = registered_fb[i]->var.yres;
-            *pFbDepth = registered_fb[i]->var.bits_per_pixel;
-            *pFbPitch = registered_fb[i]->fix.line_length;
-            break;
+            if (!registered_fb[i])
+                continue;
+
+            /* Make sure base address is mapped to GPU BAR */
+            if ((registered_fb[i]->fix.smem_start == consoleBar1Address) ||
+                (registered_fb[i]->fix.smem_start == consoleBar2Address))
+            {
+                *pPhysicalAddress = registered_fb[i]->fix.smem_start;
+                *pFbWidth = registered_fb[i]->var.xres;
+                *pFbHeight = registered_fb[i]->var.yres;
+                *pFbDepth = registered_fb[i]->var.bits_per_pixel;
+                *pFbPitch = registered_fb[i]->fix.line_length;
+                break;
+            }
+        }
+    }
+#elif NV_IS_EXPORT_SYMBOL_PRESENT_screen_info
+    /*
+     * If there is not a framebuffer console, return 0 size.
+     *
+     * orig_video_isVGA is set to 1 during early Linux kernel
+     * initialization, and then will be set to a value, such as
+     * VIDEO_TYPE_VLFB or VIDEO_TYPE_EFI if an fbdev console is used.
+     */
+    if (screen_info.orig_video_isVGA > 1)
+    {
+        NvU64 physAddr = screen_info.lfb_base;
+#if defined(VIDEO_CAPABILITY_64BIT_BASE)
+        physAddr |= (NvU64)screen_info.ext_lfb_base << 32;
+#endif
+
+        /* Make sure base address is mapped to GPU BAR */
+        if ((physAddr == consoleBar1Address) ||
+            (physAddr == consoleBar2Address))
+        {
+            *pPhysicalAddress = physAddr;
+            *pFbWidth = screen_info.lfb_width;
+            *pFbHeight = screen_info.lfb_height;
+            *pFbDepth = screen_info.lfb_depth;
+            *pFbPitch = screen_info.lfb_linelength;
         }
     }
-#else
-    *pPhysicalAddress = 0;
-    *pFbWidth = *pFbHeight = *pFbDepth = *pFbPitch = 0;
 #endif
 }
 
@@ -1295,7 +1314,7 @@ NV_STATUS NV_API_CALL os_get_euid(NvU32 *pSecToken)
 }
 
 // These functions are needed only on x86_64 platforms.
-#if (defined(NVCPU_X86_64))
+#if defined(NVCPU_X86_64)
 
 static NvBool os_verify_checksum(const NvU8 *pMappedAddr, NvU32 length)
 {
@@ -1443,13 +1462,13 @@ static NV_STATUS os_get_smbios_header_uefi(NvU64 *pSmbsAddr)
     return status;
 }
 
-#endif // #if (defined(NVCPU_X86_64))
+#endif // defined(NVCPU_X86_64)
 
 // The function locates the SMBIOS entry point.
 NV_STATUS NV_API_CALL os_get_smbios_header(NvU64 *pSmbsAddr)
 {
 
-#if (!defined(NVCPU_X86_64))
+#if !defined(NVCPU_X86_64)
     return NV_ERR_NOT_SUPPORTED;
 #else
     NV_STATUS status = NV_OK;
@@ -1530,8 +1549,7 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
     NV_STATUS status = NV_ERR_NOT_SUPPORTED;
 
 #if defined(__GFP_THISNODE) && defined(GFP_HIGHUSER_MOVABLE) && \
-    defined(__GFP_COMP) && defined(__GFP_NORETRY) && defined(__GFP_NOWARN) && \
-    defined(__GFP_ZERO)
+    defined(__GFP_COMP) && defined(__GFP_NORETRY) && defined(__GFP_NOWARN)
     gfp_t gfp_mask;
     struct page *alloc_addr;
     unsigned int order = get_order(size);
@@ -1554,7 +1572,9 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
      *                              the requested order is too large (just fail
      *                              instead).
      *
-     * 6. (Optional) __GFP_ZERO:    Used to make kernel scrub-on-allocate.
+     * 6. (Optional) __GFP_RECLAIM: Used to allow/forbid reclaim.
+     *                              This is part of GFP_USER and consequently 
+     *                              GFP_HIGHUSER_MOVABLE.
      *
      * Some of these flags are relatively more recent, with the last of them
      * (GFP_HIGHUSER_MOVABLE) having been added with this Linux kernel commit:
@@ -1568,10 +1588,12 @@ NV_STATUS NV_API_CALL os_alloc_pages_node
     gfp_mask = __GFP_THISNODE | GFP_HIGHUSER_MOVABLE | __GFP_COMP |
                __GFP_NORETRY | __GFP_NOWARN;
 
-    if (flag & NV_ALLOC_PAGES_NODE_SCRUB_ON_ALLOC)
+#if defined(__GFP_RECLAIM)
+    if (flag & NV_ALLOC_PAGES_NODE_SKIP_RECLAIM)
     {
-        gfp_mask |= __GFP_ZERO;
+        gfp_mask &= ~(__GFP_RECLAIM);
     }
+#endif // defined(__GFP_RECLAIM)
 
     alloc_addr = alloc_pages_node(nid, gfp_mask, order);
     if (alloc_addr == NULL)
@@ -1915,6 +1937,14 @@ void NV_API_CALL os_wait_uninterruptible
     wait_for_completion(&wq->q);
 }
 
+void NV_API_CALL os_wait_interruptible
+(
+    os_wait_queue *wq
+)
+{
+    wait_for_completion_interruptible(&wq->q);
+}
+
 void NV_API_CALL os_wake_up
 (
     os_wait_queue *wq
@@ -1923,39 +1953,6 @@ void NV_API_CALL os_wake_up
     complete_all(&wq->q);
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 nv_cap_t* NV_API_CALL os_nv_cap_init
 (
     const char *path
@@ -2009,93 +2006,3 @@ void NV_API_CALL os_nv_cap_close_fd
     nv_cap_close_fd(fd);
 }
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
